/*******************************************************************************
 * Copyright 2015 Julian Oppermann, TU Darmstadt
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

#ifndef EQUATIONBUILDER_H_
#define EQUATIONBUILDER_H_

#include "llvm/IR/Value.h"
#include "llvm/IR/Instructions.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/InstVisitor.h"
#include "llvm/IR/Dominators.h"

#include "Equations.h"
#include "EquationSystem.h"

using namespace llvm;

namespace cellml {
namespace opt {

class EquationBuilder : public InstVisitor<EquationBuilder> {
	EquationSystem &equationSystem;
	SymbolTable     symbolTable;

	DominatorTree   DT;

	void      insertExpr(Value *V, Expr *expr);
	void      insertIntermediateExpr(Expr *expr);
	Expr     *getExpr(Value *V);

	void      handleCallToUnaryFunction(CallInst &I, UnaryFunction function);
	void      handleCallToBinaryFunction(CallInst &I, BinaryFunction function);
	VarExpr  *handleAddress(Value *addr);
	Expr     *handlePhiPredBlocks(BasicBlock *BB, PHINode *phi);

public:
	EquationBuilder(EquationSystem &equationSystem) : equationSystem(equationSystem) {}
	void build(Function *EF);

	/* visitor interface */
	void visitBinaryOperator(BinaryOperator &I);
	void visitICmp(ICmpInst &I);
	void visitZExt(ZExtInst &I);
	void visitFCmp(FCmpInst &I);
	void visitCall(CallInst &I);
	void visitPHI(PHINode &I);
	void visitSelect(SelectInst &I);
	void visitLoad(LoadInst &I);
	void visitStore(StoreInst &I);
};

}
}

#endif /* EQUATIONBUILDER_H_ */
