/*******************************************************************************
 * Copyright 2015 Julian Oppermann, TU Darmstadt
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

#include "llvm/Pass.h"
#include "llvm/IR/Function.h"

#include "EquationBuilder.h"
#include "EquationSystem.h"

#include "CustomPasses.h"

using namespace llvm;

namespace cellml {
namespace opt {

bool EquationSystem::runOnFunction(Function &F) {
	EquationBuilder builder(*this);
	builder.build(&F);
	return false;
}

void EquationSystem::getAnalysisUsage(AnalysisUsage &AU) const {
	AU.setPreservesAll();
}

void EquationSystem::addExpr(Expr *expr) {
	exprs.insert(expr);
}

void EquationSystem::addEquation(Expr *lhs, Expr *rhs, EquationList::iterator insertBefore) {
	equations.insert(insertBefore, new Equation(lhs, rhs));
	if (lhs->getKind() == ExprKind::TmpVar)
		++nTmpVars;
}

EquationSystem::~EquationSystem()  {
	for (Equation* equation : equations)
		delete equation;
	equations.clear();

	for (Expr *expr : exprs) {
		if (expr->getSubstExpr() && exprs.count(expr->getSubstExpr()) == 0)
			delete expr->getSubstExpr();
		delete expr;
	}
	exprs.clear();
}

Pass *createEquationSystemBuilderPass() {
	return new EquationSystem();
}

unsigned Expr::lastId     = 0;
unsigned Equation::lastId = 0;

char EquationSystem::ID = 0;

}
}
