/*******************************************************************************
 * Copyright 2015 Julian Oppermann, TU Darmstadt
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

#ifndef EQUATIONSYSTEM_H_
#define EQUATIONSYSTEM_H_

#include "llvm/Pass.h"

#include "Equations.h"

using namespace llvm;

namespace cellml {
namespace opt {

class EquationSystem : public FunctionPass {
	EquationList  equations;
	ExprSet       exprs;
	unsigned      nTmpVars = 0;

public:
	static char ID;
	EquationSystem() : FunctionPass(ID) {}

	bool runOnFunction(Function &F);
	void getAnalysisUsage(AnalysisUsage &AU) const;

	void addExpr(Expr *expr);
	void addEquation(Expr *lhs, Expr *rhs, EquationList::iterator insertBefore);

	unsigned getNTmpVars() const {
		return nTmpVars;
	}

	EquationList::iterator begin() { return equations.begin(); }
	EquationList::iterator end()   { return equations.end(); }

	~EquationSystem();
};

}
}

#endif /* EQUATIONSYSTEM_H_ */
