/*******************************************************************************
 * Copyright 2015 Julian Oppermann, TU Darmstadt
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

#include "llvm/Pass.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/Attributes.h"

#include "CustomPasses.h"

using namespace llvm;

namespace cellml {
namespace opt {

class InsertNoAlias : public FunctionPass {
public:
	static char ID;
	InsertNoAlias() : FunctionPass(ID) {}

	bool runOnFunction(Function &F) {
		for (auto AI = F.arg_begin(), AE = F.arg_end(); AI != AE; AI++) {
			if (AI->getType()->isPointerTy())
				F.addAttribute(AI->getArgNo()+1, Attribute::NoAlias);
		}
		return true;
	}
};

char InsertNoAlias::ID = 0;

Pass *createNoAliasInserterPass() {
	return new InsertNoAlias();
}

}
}
