# cellml-opt - Domain-specific optimisation of CellML models

`cellml-opt` is a source-to-source compiler.
It applies a combination of standard LLVM passes and domain-specific optimisations to the equation systems found in CellML models.
Input and ouput are C files as generated by the CellML framework's C Code Generation Service.

`cellml-opt` was designed to serve as a preprocessor for the custom synthesis tool `ODoST` (developed by the Parallel and Reconfigurable Computing lab, University of Auckland) that generates fully-spatial and fully-pipelined FPGA-based accelerators for the continuous evaluation of a model's equation system.
The basic idea is to lower the resource demand after RTL synthesis by aggressively eliminating redundant computations, and replacing expensive operations (area-wise) by cheaper ones.

For more details, please refer to the following publications:
* Oppermann, J., Koch, A., Yu, T., & Sinnen, O. (2015). Domain-specific Optimisation for the High-level Synthesis of Cell Simulation Accelerators. In 25th International Conference on Field Programmable Logic and Applications, FPL 2015.
* Yu, T., Oppermann, J., Bradley, C., & Sinnen, O. (2015). Performance Optimisation Strategies for Automatically Generated FPGA Accelerators for Biomedical Models. Concurrency and Computation: Practice and Experience.

## License
cellml-opt is licensed under the Apache 2.0-License. See LICENSE.txt for more information. 

## Prerequisites
You'll need a Unix-like system with a C++11 compatible toolchain, Python 2.x and CMake.

## Setup
* Get LLVM and switch to the 3.5 release:  
	`$ cd /some/path`  
	`$ git clone http://llvm.org/git/llvm.git`  
	`$ cd /some/path/llvm`  
	`$ git checkout -t origin/release_35`  
	`$ cd /some/path/llvm/tools`  
	`$ git clone http://llvm.org/git/clang.git`  
	`$ cd /some/path/llvm/tools/clang`  
	`$ git checkout -t origin/release_35`

* Build LLVM:  
	`$ cd /some/path/llvm`  
	`$ mkdir build && cd build`  
	`$ cmake -DCMAKE_BUILD_TYPE=Debug ..`  
	`$ make`  

* Come back to this repository.  

* Build cellml-opt:  
	`$ mkdir build && cd build`  
	`$ cmake -DLLVM_DIR=/some/path/llvm/build/share/llvm/cmake -DCMAKE_BUILD_TYPE=Debug ..`  
	`$ make`  

* Create a configuration file `config.py` containing the paths to the `clang`, `opt` and `cellml-opt` binaries.  
	Example:  
	`CLANG = /some/path/llvm/build/bin/clang`  
	`OPT = /some/path/llvm/build/bin/opt`  
	`CELLML_OPT = /another/path/cellml-opt/build/cellml-opt`

## Usage
Use the supplied driver script `cellml-opt.py` to optimise a model's C code. Refer to the output of `cellml-opt --help` to learn about the available command line options. The optimisation presets/compilation flows are explained in the aforementioned FPL paper in more detail.

#### Example
`cellml-opt.py -OEqC model.c -o model.smaller.c` applies the compilation flow containing standard and custom optimisations to the model and emits the reconstructed equation system into the file `model.smaller.c`.

To compare the results of the different optimisation presets, and to use the generic driver to evaluate the result accuracy, you can use the rules in `tests/Makefile.common`.

#### Example
`cd tests/example/cellml/example`  
`make` Optimises the model in example.c with four different presets. Statistics are available in the files `<preset>.log`, e.g. `EqC.log`.  
`make run` Evaluates the different versions of the model's equation system with the generic driver. Every variable in the model is dumped after each evaluation step; their values are compared numerically. Inspect the file `errors.out` to see how the different flows affected the result accuracy.

Real models can be found in the CellML repository: https://models.physiomeproject.org.
For example, https://models.physiomeproject.org/e/c1/beeler_reuter_1977.cellml/@@cellml_codegen/C is a model that has been investigated in the FPL paper.

## Implementation
* Julian Oppermann, Embedded Systems and Applications Group, Technische Universität Darmstadt <oppermann@esa.tu-darmstadt.de>

## Contributors
* Ting Yu,          Parallel and Reconfigurable Computing Group, University of Auckland
* Oliver Sinnen,    Parallel and Reconfigurable Computing Group, University of Auckland
* Andreas Koch,     Embedded Systems and Applications Group, Technische Universität Darmstadt

