################################################################################
# Copyright 2015 Julian Oppermann, TU Darmstadt
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
################################################################################
#
# calc-errors.py -- Simple result comparison tool for the output of 
#                   CellML model solvers.
#
################################################################################

import optparse
import string

def readFile(file):
	lines = [line.strip() for line in open(file)]
	return [float(line) for line in lines[2:]]

def main():
	usage = "usage: %prog <ref> <out1> [outN...]"
	parser = optparse.OptionParser(usage)
	parser.add_option("--opt-levels", dest="opt_levels", help="specify optlevels corresponding to the input files")
	(options, args) = parser.parse_args()
	
	if len(args) < 2:
		parser.error("need at least two files to compare")
	
	optLevels = None
	if options.opt_levels:
		optLevels = string.split(options.opt_levels, " ")
		if (len(optLevels) != len(args)):
			parser.error("file <-> opt level mismatch")
	
	refFloats = readFile(args[0])
	outFloats = [readFile(outFile) for outFile in args[1:]] 
	
	maxErrorAbs = [0.0 for x in range(len(outFloats))]
	maxErrorRel = maxErrorAbs[:]
	
	W = 15
		
	for i, r in enumerate(refFloats):
		cmp = ("%f" % r).rjust(W) + " | "
		for j, of in enumerate(outFloats):
			value = of[i]
			errorAbs = abs(value - r)
			errorRel = errorAbs / (abs(r) or 1)
			
			if (errorRel > maxErrorRel[j]):
				maxErrorAbs[j] = errorAbs
				maxErrorRel[j] = errorRel
			
			marker = None
			if of[i] != r:
				marker = " < "
			
			cmp += ("%.9f"    % (value)).rjust(W) + " "
			cmp += ("%.9f"    % (errorAbs)).rjust(W) + " "
			cmp += ("%.9f %%" % (errorRel * 100)).rjust(W) + (marker or " | ")
			
		print cmp
	
	errors = "Max errors: ".ljust(W) + " | "
	for i, e in enumerate(maxErrorAbs):
		marker = None
		if e != 0.0:
			marker = " < "
		
		optLevel = ""
		if optLevels:
			optLevel = optLevels[i+1]
		
		errors += (optLevel).ljust(W) + " "
		errors += ("%.9f"    % (maxErrorAbs[i])).rjust(W) + " "
		errors += ("%.9f %%" % (maxErrorRel[i] * 100)).rjust(W) + (marker or " | ")
	
	print errors
	
	if optLevels:
		for i, optLevel in enumerate(optLevels):
			statsFile = open("stats.%s.csv" % optLevel, "r+")
			lines = statsFile.readlines()
			assert len(lines) == 1
			statsFile.seek(0)
			if i == 0: # the reference mode
				statsFile.write("%s,%.9f,%.9f\n" % (lines[0].strip(), 0.0, 0.0))
			else:
				statsFile.write("%s,%.9f,%.9f\n" % (lines[0].strip(), maxErrorAbs[i-1], maxErrorRel[i-1] * 100))
			statsFile.close()

if __name__ == "__main__":
	main()
