################################################################################
# Copyright 2015 Julian Oppermann, TU Darmstadt
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
################################################################################
#
# emit-defines.py -- Helper to extract the model dimensions into a
#                    C header file.
#
################################################################################

import optparse
import re

usage = "usage: %prog [options] <CellML model>"
parser = optparse.OptionParser(usage)
parser.add_option("-o", dest="outfile", help="specify output filename")
(options, args) = parser.parse_args()

f = open(args[0], "r")
s = f.read()
f.close()

p = re.compile(r"""
\s*There are a total of (\d+) entries in the algebraic variable array.
\s*There are a total of (\d+) entries in each of the rate and state variable arrays.
\s*There are a total of (\d+) entries in the constant variable array.
"""
)
m = re.search(p, s)
if m != None:
	(nAlgebraic, nStatesRates, nConstants) = m.groups()
	
f = open(options.outfile or "model.h", "w")
f.write("""
#define NUM_STATES    %s
#define NUM_RATES     %s
#define NUM_CONSTANTS %s
#define NUM_ALGEBRAIC %s
""" % (nStatesRates, nStatesRates, nConstants, nAlgebraic))
f.close()
